import { createEffect, onCleanup } from "solid-js";
import { createStore } from "solid-js/store";
import * as pbf from "@nedpals/pbf";
import { usePB } from "./context.js";
export function useRecord(collection, id, options) {
    const pb = usePB();
    const requestKey = crypto.randomUUID();
    const [record, setRecord] = createStore({
        loading: true,
        error: null,
        record: null,
    });
    const v = options?.validator ?? ({ parse: (i) => id });
    createEffect(() => {
        const cid = id();
        const setRecordChecked = (...args) => {
            if (cid !== id())
                return;
            setRecord(...args);
        };
        setRecordChecked("loading", () => true);
        setRecordChecked("error", () => null);
        pb.collection(collection).getOne(cid, {
            expand: options?.expand ? options.expand.join(",") : undefined,
            requestKey,
        }).then((value) => {
            setRecordChecked("record", () => v.parse(value));
        }).catch((error) => {
            setRecordChecked("error", () => error);
        }).finally(() => {
            setRecordChecked("loading", () => false);
        });
        const sub = pb.realtime.subscribe(collection + "/" + cid, (value) => {
            //TODO: Check what actions exist and if some needs special handling
            setRecordChecked("record", () => v.parse(value.record));
        }, {
            query: {
                expand: options?.expand ? options.expand.join(",") : undefined,
                requestKey: requestKey + "-rt",
            }
        });
        sub.catch(err => { console.error("Error in realtime subscription", err); });
        onCleanup(() => {
            pb.cancelRequest(requestKey);
            pb.cancelRequest(requestKey + "-rt");
            sub.then(s => s());
        });
    });
    return record;
}
export function useRecords(collection, options = { expand: [] }) {
    const pb = usePB();
    const requestKey = crypto.randomUUID();
    const fstr = () => options.filter && options.filter() ? pbf.stringify(options.filter()) : undefined;
    const v = options.validator ?? ({ parse: (i) => i });
    const [records, setRecords] = createStore({
        records: [],
        loading: true,
        error: null,
    });
    createEffect(() => {
        const cf = fstr();
        const setRecordChecked = (...args) => {
            if (cf !== fstr())
                return;
            setRecords(...args);
        };
        setRecordChecked("loading", () => true);
        setRecordChecked("error", () => null);
        pb.collection(collection).getFullList({
            requestKey,
            filter: cf,
            expand: options?.expand ? options.expand.join(",") : undefined,
        }).then((values) => {
            setRecordChecked("records", () => values.map(value => v.parse(value)));
        }).catch((error) => {
            setRecordChecked("error", () => error);
        }).finally(() => {
            setRecordChecked("loading", () => false);
        });
        const sub = pb.realtime.subscribe(collection, (event) => {
            console.log("Event", event);
            const rec = v.parse(event.record);
            switch (event.action) {
                case "create":
                    setRecordChecked("records", (records) => [...records, rec]);
                    break;
                case "update":
                    setRecordChecked("records", (records) => records.map(r => r.id === rec.id ? rec : r));
                    break;
                case "delete":
                    setRecordChecked("records", (records) => records.filter(r => r.id !== rec.id));
                    break;
                default:
                    console.log("Unknown action", event.action);
            }
        }, {
            requestKey: requestKey + "-rt",
            query: {
                filter: cf,
                expand: options?.expand ? options.expand.join(",") : undefined,
            }
        });
        sub.then(unsub => { console.log("Subscribed to realtime changes!"); });
        sub.catch(err => { console.error("Error in realtime subscription", err); });
        onCleanup(() => {
            pb.cancelRequest(requestKey);
            pb.cancelRequest(requestKey + "-rt");
            sub.then(s => s());
        });
    });
    return records;
}
